package proxy.util;

import java.nio.charset.Charset;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.util.Collection;
import com.google.common.collect.Lists;
import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;

public class User {
	private static final Charset UTF_8 = Charset.forName("UTF-8");
	
	protected final String username;
	
	private byte[] hash;
	private byte[] salt;
	protected boolean admin;
	protected String skinTexture;
	protected String skinSignature;
	
	public User(String username) {
		this.username = username;
	}
	
	public void setPassword(byte[] hash, byte[] salt) {
		this.hash = hash;
		this.salt = salt;
	}
	
	public String getUsername() {
		return this.username;
	}
	
	public boolean checkPassword(String password) {
		return MessageDigest.isEqual(hashPassword(password, this.salt), this.hash);
	}
	
	public boolean isAdmin() {
		return this.admin;
	}
	
	public boolean hasPermission(String perm) {
		return this.isAdmin();
	}
	
	public void setAdmin(boolean admin) {
		this.admin = admin;
	}
	
	public void setSkinTexture(String data, String signature) {
		this.skinTexture = data;
		this.skinSignature = signature;
	}
	
	public boolean isOnline() {
		return false;
	}
	
	public void copyFrom(User user) {
		this.hash = user.hash;
		this.salt = user.salt;
		this.admin = user.admin;
		this.skinTexture = user.skinTexture;
		this.skinSignature = user.skinSignature;
	}
	
	private JsonObject toJson() {
		JsonObject obj = new JsonObject();
		obj.addProperty("username", this.username);
		obj.addProperty("passwordHash", Formatter.getHexString(this.hash));
		obj.addProperty("passwordSalt", Formatter.getHexString(this.salt));
		obj.addProperty("admin", this.admin);
		obj.addProperty("skinTexture", this.skinTexture);
		obj.addProperty("skinSignature", this.skinSignature);
		return obj;
	}
	
	private boolean fromJson(JsonObject obj) {
		if(!Formatter.isValidUser(this.username))
			return false;
		this.hash = obj.has("passwordHash") ? Formatter.fromHexString(obj.get("passwordHash").getAsString()) : null;
		this.salt = obj.has("passwordSalt") ? Formatter.fromHexString(obj.get("passwordSalt").getAsString()) : null;
		this.admin = obj.has("admin") ? obj.get("admin").getAsBoolean() : false;
		this.skinTexture = obj.has("skinTexture") ? obj.get("skinTexture").getAsString() : null;
		this.skinSignature = obj.has("skinSignature") ? obj.get("skinSignature").getAsString() : null;
		return this.hash != null && this.salt != null;
	}
	
	public static JsonArray toJson(Collection<User> users) {
		JsonArray arr = new JsonArray();
		for(User user : users) {
			arr.add(user.toJson());
		}
		return arr;
	}
	
	public static Collection<User> fromJson(JsonArray arr) {
		Collection<User> users = Lists.newArrayList();
		for(JsonElement elem : arr) {
			if(!elem.isJsonObject())
				continue;
			JsonObject obj = elem.getAsJsonObject();
			if(!obj.has("username"))
				continue;
			JsonElement name = obj.get("username");
			if(!name.isJsonPrimitive())
				continue;
			User user = new User(name.getAsString());
			if(user.fromJson(obj))
				users.add(user);
		}
		return users;
	}
	
	private static byte[] hashPassword(String pass, byte[] salt) {
		try {
			MessageDigest digest = MessageDigest.getInstance("SHA-512");
			digest.update(pass.getBytes(UTF_8));
			digest.update(salt);
			return digest.digest();
		}
		catch(NoSuchAlgorithmException e) {
			Log.error("Could not generate password hash");
			return null;
		}
	}
	
	public static User createUser(String user, String pass) {
		SecureRandom rand = new SecureRandom();
		byte[] salt = new byte[32];
		rand.nextBytes(salt);
		User usr = new User(user);
		usr.setPassword(hashPassword(pass, salt), salt);
		return usr;
	}
}
